@@ -9,7 +9,7 @@
 
 from vdirsyncer.storage.base import Item
 import vdirsyncer.exceptions as exceptions
-from .. import assert_item_equals, normalize_item
+from .. import assert_item_equals
 import random
 import pytest
 
@@ -48,6 +48,10 @@ class StorageTests(object):
             assert etag == etag2
             assert 'UID:{}'.format(item.uid) in item.raw
 
+    def test_empty_get_multi(self):
+        s = self._get_storage()
+        assert list(s.get_multi([])) == []
+
     def test_upload_already_existing(self):
         s = self._get_storage()
         item = self._create_bogus_item(1)
@@ -88,6 +92,12 @@ class StorageTests(object):
         with pytest.raises(exceptions.PreconditionFailed):
             s.delete(href, '"lolnope"')
 
+    def test_delete(self):
+        s = self._get_storage()
+        href, etag = s.upload(self._create_bogus_item(1))
+        s.delete(href, etag)
+        assert not list(s.list())
+
     def test_delete_nonexisting(self):
         s = self._get_storage()
         with pytest.raises(exceptions.PreconditionFailed):
@@ -134,8 +144,22 @@ class StorageTests(object):
 
         assert not collections
 
+    def test_discover_collection_arg(self):
+        args = self.get_storage_args(collection='test2')
+        with pytest.raises(TypeError) as excinfo:
+            list(self.storage_class.discover(**args))
+
+        assert 'collection argument must not be given' in str(excinfo.value)
+
     def test_collection_arg(self):
         s = self.storage_class(**self.get_storage_args(collection='test2'))
         # Can't do stronger assertion because of radicale, which needs a
         # fileextension to guess the collection type.
         assert 'test2' in s.collection
+
+    def test_has(self):
+        s = self._get_storage()
+        assert not s.has('asd')
+        href, etag = s.upload(self._create_bogus_item(1))
+        assert s.has(href)
+        assert not s.has('asd')
