@@ -7,16 +7,15 @@
     :license: MIT, see LICENSE for more details.
 '''
 
-from .. import assert_item_equals
 from textwrap import dedent
-from vdirsyncer.storage.http import HttpStorage, Item
+from vdirsyncer.storage.http import HttpStorage
 from requests import Response
 
 
 class TestHttpStorage(object):
 
     def test_list(self, monkeypatch):
-        collection_url = 'http://127.0.0.1/calendar/collection/'
+        collection_url = 'http://127.0.0.1/calendar/collection.ics'
 
         items = [
             dedent(b'''
@@ -44,14 +43,16 @@ class TestHttpStorage(object):
             '\n'.join([b'BEGIN:VCALENDAR'] + items + [b'END:VCALENDAR'])
         ] * 2
 
-        def get(*a, **kw):
+        def get(method, url, *a, **kw):
+            assert method == 'GET'
+            assert url == collection_url
             r = Response()
             r.status_code = 200
             assert responses
             r._content = responses.pop()
             return r
 
-        monkeypatch.setattr('requests.get', get)
+        monkeypatch.setattr('requests.request', get)
 
         s = HttpStorage(url=collection_url)
 
