@@ -116,12 +116,15 @@ def test_already_synced():
     old_status = dict(status)
     a.update = b.update = a.upload = b.upload = \
         lambda *a, **kw: pytest.fail('Method shouldn\'t have been called.')
-    sync(a, b, status)
-    assert status == old_status
-    assert a.has('1.txt') and b.has('1.txt')
+
+    for i in (1, 2):
+        sync(a, b, status)
+        assert status == old_status
+        assert a.has('1.txt') and b.has('1.txt')
 
 
-def test_conflict_resolution_both_etags_new():
+@pytest.mark.parametrize('winning_storage', 'ab')
+def test_conflict_resolution_both_etags_new(winning_storage):
     a = MemoryStorage()
     b = MemoryStorage()
     item = Item(u'UID:1')
@@ -130,17 +133,28 @@ def test_conflict_resolution_both_etags_new():
     status = {}
     sync(a, b, status)
     assert status
-    a.update(href_a, Item(u'UID:1\nASDASD'), etag_a)
-    b.update(href_b, Item(u'UID:1\nHUEHUE'), etag_b)
+    a.update(href_a, Item(u'UID:1\nitem a'), etag_a)
+    b.update(href_b, Item(u'UID:1\nitem b'), etag_b)
     with pytest.raises(exceptions.SyncConflict):
         sync(a, b, status)
-    sync(a, b, status, conflict_resolution='a wins')
+    sync(a, b, status, conflict_resolution='{} wins'.format(winning_storage))
     item_a, _ = a.get(href_a)
     item_b, _ = b.get(href_b)
     assert_item_equals(item_a, item_b)
     n = normalize_item(item_a)
     assert u'UID:1' in n
-    assert u'ASDASD' in n
+    assert u'item {}'.format(winning_storage) in n
+
+
+def test_conflict_resolution_invalid_mode():
+    a = MemoryStorage()
+    b = MemoryStorage()
+    item_a = Item(u'UID:1\nitem a')
+    item_b = Item(u'UID:1\nitem b')
+    a.upload(item_a)
+    b.upload(item_b)
+    with pytest.raises(ValueError):
+        sync(a, b, {}, conflict_resolution='yolo')
 
 
 def test_conflict_resolution_new_etags_without_changes():
@@ -152,3 +166,28 @@ def test_conflict_resolution_new_etags_without_changes():
     status = {'1': (href_a, 'BOGUS_a', href_b, 'BOGUS_b')}
     sync(a, b, status)
     assert status == {'1': (href_a, etag_a, href_b, etag_b)}
+
+def test_uses_get_multi(monkeypatch):
+    def breakdown(*a, **kw):
+        raise AssertionError('Expected use of get_multi')
+
+    get_multi_calls = []
+
+    old_get = MemoryStorage.get
+
+    def get_multi(self, hrefs):
+        get_multi_calls.append(hrefs)
+        for href in hrefs:
+            item, etag = old_get(self, href)
+            yield href, item, etag
+
+    monkeypatch.setattr(MemoryStorage, 'get', breakdown)
+    monkeypatch.setattr(MemoryStorage, 'get_multi', get_multi)
+
+    a = MemoryStorage()
+    b = MemoryStorage()
+    item = Item(u'UID:1')
+    expected_href, etag = a.upload(item)
+
+    sync(a, b, {})
+    assert get_multi_calls == [[expected_href]]
