@@ -21,7 +21,9 @@ def test_parse_options():
         'asd': 'off'
     }
 
-    assert dict(utils.parse_options(o.items())) == {
+    a = dict(utils.parse_options(o.items()))
+
+    expected = {
         'foo': True,
         'hah': True,
         'bar': '',
@@ -30,6 +32,14 @@ def test_parse_options():
         'asd': False
     }
 
+    assert a == expected
+
+    for key in a:
+        # Yes, we want a very strong typecheck here, because we actually have
+        # to differentiate between bool and int, and in Python 2, bool is a
+        # subclass of int.
+        assert type(a[key]) is type(expected[key])
+
 
 def test_get_password_from_netrc(monkeypatch):
     username = 'foouser'
