@@ -7,10 +7,11 @@
     :license: MIT, see LICENSE for more details.
 '''
 
-from .base import Item
-from .http import HttpStorageBase
-import vdirsyncer.exceptions as exceptions
-import vdirsyncer.log as log
+from .base import Storage, Item
+from .http import prepare_auth, prepare_verify, USERAGENT
+from .. import exceptions
+from .. import log
+from ..utils import request, get_password
 import requests
 import datetime
 import urlparse
@@ -23,7 +24,7 @@ CALDAV_DT_FORMAT = '%Y%m%dT%H%M%SZ'
 CONFIG_DT_FORMAT = '%Y-%m-%d'
 
 
-class DavStorage(HttpStorageBase):
+class DavStorage(Storage):
 
     # the file extension of items. Useful for testing against radicale.
     fileext = None
@@ -41,11 +42,40 @@ class DavStorage(HttpStorageBase):
     leif_class = None
 
     _session = None
-    _repr_attributes = ('url', 'username')
+    _repr_attributes = ('username', 'url')
+
+    def __init__(self, url, username='', password='', collection=None,
+                 verify=True, auth=None, useragent=USERAGENT, **kwargs):
+        '''
+        :param url: Base URL or an URL to a collection. Autodiscovery should be
+            done via :py:meth:`DavStorage.discover`.
+        :param username: Username for authentication.
+        :param password: Password for authentication.
+        :param verify: Verify SSL certificate, default True.
+        :param auth: Authentication method, from {'basic', 'digest'}, default
+            'basic'.
+        :param useragent: Default 'vdirsyncer'.
+        '''
 
-    def __init__(self, **kwargs):
         super(DavStorage, self).__init__(**kwargs)
 
+        if username and not password:
+            password = get_password(username, url)
+
+        self._settings = {
+            'verify': prepare_verify(verify),
+            'auth': prepare_auth(auth, username, password)
+        }
+        self.username, self.password = username, password
+        self.useragent = useragent
+
+        url = url.rstrip('/') + '/'
+        if collection is not None:
+            url = urlparse.urljoin(url, collection)
+        self.url = url.rstrip('/') + '/'
+        self.parsed_url = urlparse.urlparse(self.url)
+        self.collection = collection
+
         headers = self._default_headers()
         headers['Depth'] = 1
         response = self._request(
@@ -57,6 +87,12 @@ class DavStorage(HttpStorageBase):
         if self.dav_header not in response.headers.get('DAV', ''):
             raise exceptions.StorageError('URL is not a collection')
 
+    def _default_headers(self):
+        return {
+            'User-Agent': self.useragent,
+            'Content-Type': 'application/xml; charset=UTF-8'
+        }
+
     @classmethod
     def discover(cls, url, **kwargs):
         if kwargs.pop('collection', None) is not None:
@@ -93,13 +129,15 @@ class DavStorage(HttpStorageBase):
         if self._session is None:
             self._session = requests.session()
         url = self.parsed_url.scheme + '://' + self.parsed_url.netloc + path
-        return self._session.request(method, url, data=data, headers=headers,
-                                     **self._settings)
+        return request(method, url, data=data, headers=headers,
+                       session=self._session, **self._settings)
 
     @staticmethod
     def _check_response(response):
         if response.status_code == 412:
             raise exceptions.PreconditionFailed(response.reason)
+        if response.status_code == 404:
+            raise exceptions.NotFoundError(response.reason)
         response.raise_for_status()
 
     def get(self, href):
@@ -122,7 +160,7 @@ class DavStorage(HttpStorageBase):
             data=data,
             headers=self._default_headers()
         )
-        response.raise_for_status()
+        self._check_response(response)
         root = etree.XML(response.content)  # etree only can handle bytes
         rv = []
         hrefs_left = set(hrefs)
@@ -151,14 +189,6 @@ class DavStorage(HttpStorageBase):
             raise exceptions.NotFoundError(href)
         return rv
 
-    def has(self, href):
-        try:
-            self.get(href)
-        except exceptions.PreconditionFailed:
-            return False
-        else:
-            return True
-
     def _put(self, href, item, etag):
         headers = self._default_headers()
         headers['Content-Type'] = self.item_mimetype
@@ -204,16 +234,28 @@ class DavStorage(HttpStorageBase):
             href,
             headers=headers
         )
-        if response.status_code == 404:
-            raise exceptions.NotFoundError(href)
         self._check_response(response)
 
     def _list(self, xml):
+        headers = self._default_headers()
+
+        # CardDAV: The request MUST include a Depth header. The scope of the
+        # query is determined by the value of the Depth header. For example,
+        # to query all address object resources in an address book collection,
+        # the REPORT would use the address book collection as the Request- URI
+        # and specify a Depth of 1 or infinity.
+        # http://tools.ietf.org/html/rfc6352#section-8.6
+        #
+        # CalDAV:
+        # The request MAY include a Depth header.  If no Depth header is
+        # included, Depth:0 is assumed.
+        # http://tools.ietf.org/search/rfc4791#section-7.8
+        headers['Depth'] = 'infinity'
         response = self._request(
             'REPORT',
             '',
             data=xml,
-            headers=self._default_headers()
+            headers=headers
         )
         response.raise_for_status()
         root = etree.XML(response.content)
@@ -247,14 +289,16 @@ class CaldavStorage(DavStorage):
     get_multi_data_query = '{urn:ietf:params:xml:ns:caldav}calendar-data'
 
     def __init__(self, start_date=None, end_date=None,
-                 item_types=('VTODO', 'VEVENT'), **kwargs):
+                 item_types=(), **kwargs):
         '''
         :param start_date: Start date of timerange to show, default -inf.
         :param end_date: End date of timerange to show, default +inf.
-        :param item_types: The item types to show from the server. Dependent on
-            server functionality, no clientside validation of results. This
-            currently only affects the `list` method, but this shouldn't cause
-            problems in the normal usecase.
+        :param item_types: A tuple of collection types to show from the server.
+            For example, if you want to only get VEVENTs, pass ``('VEVENT',)``.
+            Falsy values mean "get all types". Dependent on server
+            functionality, no clientside validation of results. This currently
+            only affects the `list` method, but this shouldn't cause problems
+            in the normal usecase.
         '''
         super(CaldavStorage, self).__init__(**kwargs)
         if isinstance(item_types, str):
@@ -272,9 +316,34 @@ class CaldavStorage(DavStorage):
                 (eval(end_date, namespace) if isinstance(end_date, str)
                  else end_date)
 
-        self._list_template = self._get_list_template()
+    @staticmethod
+    def _get_list_filters(components, start, end):
 
-    def _get_list_template(self):
+        if not components:
+            components = ('VTODO', 'VEVENT')
+
+        caldavfilter = '''
+            <C:comp-filter name="VCALENDAR">
+                <C:comp-filter name="{component}">
+                    {timefilter}
+                </C:comp-filter>
+            </C:comp-filter>
+            '''
+
+        if start is not None and end is not None:
+            start = start.strftime(CALDAV_DT_FORMAT)
+            end = end.strftime(CALDAV_DT_FORMAT)
+
+            timefilter = ('<C:time-range start="{start}" end="{end}"/>'
+                .format(start=start, end=end))
+        else:
+            timefilter = ''
+
+        for component in components:
+            yield caldavfilter.format(component=component,
+                                      timefilter=timefilter)
+
+    def list(self):
         data = '''<?xml version="1.0" encoding="utf-8" ?>
             <C:calendar-query xmlns:D="DAV:"
                 xmlns:C="urn:ietf:params:xml:ns:caldav">
@@ -282,27 +351,17 @@ class CaldavStorage(DavStorage):
                     <D:getetag/>
                 </D:prop>
                 <C:filter>
-                    <C:comp-filter name="VCALENDAR">
-                        <C:comp-filter name="{component}">
-                            {caldavfilter}
-                        </C:comp-filter>
-                    </C:comp-filter>
+                {caldavfilter}
                 </C:filter>
             </C:calendar-query>'''
-        start = self.start_date
-        end = self.end_date
-        caldavfilter = ''
-        if start is not None and end is not None:
-            start = start.strftime(CALDAV_DT_FORMAT)
-            end = end.strftime(CALDAV_DT_FORMAT)
-            caldavfilter = ('<C:time-range start="{start}" end="{end}"/>'
-                            .format(start=start, end=end))
-        return data.format(caldavfilter=caldavfilter, component='{item_type}')
 
-    def list(self):
         hrefs = set()
-        for t in self.item_types:
-            xml = self._list_template.format(item_type=t)
+
+        caldavfilters = self._get_list_filters(self.item_types,
+                                               self.start_date, self.end_date)
+
+        for caldavfilter in caldavfilters:
+            xml = data.format(caldavfilter=caldavfilter)
             for href, etag in self._list(xml):
                 assert href not in hrefs
                 hrefs.add(href)
