@@ -19,6 +19,34 @@ def _get_etag(fpath):
     return '{:.9f}'.format(os.path.getmtime(fpath))
 
 
+class safe_write(object):
+    f = None
+    tmppath = None
+    fpath = None
+    mode = None
+
+    def __init__(self, fpath, mode):
+        self.tmppath = fpath + '.tmp'
+        self.fpath = fpath
+        self.mode = mode
+
+    def __enter__(self):
+        self.f = f = open(self.tmppath, self.mode)
+        self.write = f.write
+        return self
+
+    def __exit__(self, type, value, tb):
+        self.f.close()
+        if type is None:
+            os.rename(self.tmppath, self.fpath)
+        else:
+            os.remove(self.tmppath)
+
+    def get_etag(self):
+        self.f.flush()
+        return _get_etag(self.tmppath)
+
+
 class FilesystemStorage(Storage):
 
     '''Saves data in vdir collection
@@ -41,6 +69,7 @@ class FilesystemStorage(Storage):
         :param create: Create directories if they don't exist.
         '''
         super(FilesystemStorage, self).__init__(**kwargs)
+        path = expand_path(path)
         if collection is not None:
             path = os.path.join(path, collection)
         if not os.path.isdir(path):
@@ -54,7 +83,7 @@ class FilesystemStorage(Storage):
                               'create it, or create it '
                               'yourself.'.format(path))
         self.collection = collection
-        self.path = expand_path(path)
+        self.path = path
         self.encoding = encoding
         self.fileext = fileext
 
@@ -80,21 +109,25 @@ class FilesystemStorage(Storage):
 
     def get(self, href):
         fpath = self._get_filepath(href)
-        with open(fpath, 'rb') as f:
-            return (Item(f.read().decode(self.encoding)),
-                    _get_etag(fpath))
-
-    def has(self, href):
-        return os.path.isfile(self._get_filepath(href))
+        try:
+            with open(fpath, 'rb') as f:
+                return (Item(f.read().decode(self.encoding)),
+                        _get_etag(fpath))
+        except IOError as e:
+            import errno
+            if e.errno == errno.ENOENT:
+                raise exceptions.NotFoundError(href)
+            else:
+                raise
 
     def upload(self, item):
         href = self._get_href(item.uid)
         fpath = self._get_filepath(href)
         if os.path.exists(fpath):
             raise exceptions.AlreadyExistingError(item.uid)
-        with open(fpath, 'wb+') as f:
+        with safe_write(fpath, 'wb+') as f:
             f.write(item.raw.encode(self.encoding))
-        return href, _get_etag(fpath)
+            return href, f.get_etag()
 
     def update(self, href, item, etag):
         fpath = self._get_filepath(href)
@@ -107,9 +140,9 @@ class FilesystemStorage(Storage):
         if etag != actual_etag:
             raise exceptions.WrongEtagError(etag, actual_etag)
 
-        with open(fpath, 'wb') as f:
-            f.write(item.raw.encode('utf-8'))
-        return _get_etag(fpath)
+        with safe_write(fpath, 'wb') as f:
+            f.write(item.raw.encode(self.encoding))
+            return f.get_etag()
 
     def delete(self, href, etag):
         fpath = self._get_filepath(href)
