@@ -8,11 +8,12 @@
     :license: MIT, see LICENSE for more details.
 '''
 
-import requests
 import urlparse
 import hashlib
 from .base import Storage, Item
-from vdirsyncer.utils import expand_path, get_password
+from vdirsyncer.utils import expand_path, get_password, request
+
+USERAGENT = 'vdirsyncer'
 
 
 def split_collection(text):
@@ -58,16 +59,16 @@ def prepare_auth(auth, username, password):
 
 
 def prepare_verify(verify):
-    if isinstance(verify, bool):
-        return verify
-    return expand_path(verify)
+    if isinstance(verify, (str, unicode)):
+        return expand_path(verify)
+    return verify
 
 
-class HttpStorageBase(Storage):
+class HttpStorage(Storage):
     _repr_attributes = ('username', 'url')
 
     def __init__(self, url, username='', password='', collection=None,
-                 verify=True, auth=None, useragent='vdirsyncer', **kwargs):
+                 verify=True, auth=None, useragent=USERAGENT, **kwargs):
         '''
         :param url: Base URL or an URL to a collection. Autodiscovery should be
             done via :py:meth:`DavStorage.discover`.
@@ -78,7 +79,7 @@ class HttpStorageBase(Storage):
             'basic'.
         :param useragent: Default 'vdirsyncer'.
         '''
-        super(HttpStorageBase, self).__init__(**kwargs)
+        super(HttpStorage, self).__init__(**kwargs)
 
         if username and not password:
             password = get_password(username, url)
@@ -90,30 +91,18 @@ class HttpStorageBase(Storage):
         self.username, self.password = username, password
         self.useragent = useragent
 
-        url = url.rstrip('/') + '/'
         if collection is not None:
             url = urlparse.urljoin(url, collection)
-        self.url = url.rstrip('/') + '/'
+        self.url = url
         self.parsed_url = urlparse.urlparse(self.url)
         self.collection = collection
-
-    def _default_headers(self):
-        return {
-            'User-Agent': self.useragent,
-            'Content-Type': 'application/xml; charset=UTF-8'
-        }
-
-
-class HttpStorage(HttpStorageBase):
-
-    _items = None
-
-    def __init__(self, **kwargs):
-        super(HttpStorage, self).__init__(**kwargs)
         self._items = {}
 
+    def _default_headers(self):
+        return {'User-Agent': self.useragent}
+
     def list(self):
-        r = requests.get(self.url, **self._settings)
+        r = request('GET', self.url, **self._settings)
         r.raise_for_status()
         self._items.clear()
         for i, item in enumerate(split_collection(r.text)):
@@ -126,6 +115,3 @@ class HttpStorage(HttpStorageBase):
     def get(self, href):
         x = self._items[href]
         return x, hashlib.sha256(x.raw).hexdigest()
-
-    def has(self, href):
-        return href in self._items
