@@ -9,35 +9,36 @@
     type. This allows us not only to synchronize a local vdir with a CalDAV
     server, but also synchronize two CalDAV servers or two local vdirs.
 
+    The algorithm is based on the blogpost "How OfflineIMAP works" by Edward Z.
+    Yang. http://blog.ezyang.com/2012/08/how-offlineimap-works/
+
     :copyright: (c) 2014 Markus Unterwaditzer
     :license: MIT, see LICENSE for more details.
 '''
+import itertools
+
 import vdirsyncer.exceptions as exceptions
 import vdirsyncer.log
 sync_logger = vdirsyncer.log.get('sync')
 
 
-def prepare_list(storage, href_to_uid):
+def prepare_list(storage, href_to_status):
+    download = []
     for href, etag in storage.list():
         props = {'etag': etag}
-        if href in href_to_uid:
-            props['uid'] = href_to_uid[href]
+        if href in href_to_status:
+            uid, old_etag = href_to_status[href]
+            props['uid'] = uid
+            if etag != old_etag:
+                download.append(href)
         else:
-            item, new_etag = storage.get(href)
-            assert etag == new_etag
-            props['uid'] = item.uid
-            props['item'] = item
+            download.append(href)
         yield href, props
 
-
-def prefetch(storage, item_list, hrefs):
-    hrefs_to_prefetch = []
-    for href in hrefs:
-        if 'item' not in item_list[href]:
-            hrefs_to_prefetch.append(href)
-    for href, item, etag in storage.get_multi(hrefs_to_prefetch):
-        assert item_list[href]['etag'] == etag
-        item_list[href]['item'] = item
+    if download:
+        for href, item, etag in storage.get_multi(download):
+            props = {'item': item, 'uid': item.uid, 'etag': etag}
+            yield href, props
 
 
 def sync(storage_a, storage_b, status, conflict_resolution=None):
@@ -55,34 +56,29 @@ def sync(storage_a, storage_b, status, conflict_resolution=None):
         provided, the sync function will raise
         :py:exc:`vdirsyncer.exceptions.SyncConflict`.
     '''
-    a_href_to_uid = dict(
-        (href_a, uid)
+    a_href_to_status = dict(
+        (href_a, (uid, etag_a))
         for uid, (href_a, etag_a, href_b, etag_b) in status.iteritems()
     )
-    b_href_to_uid = dict(
-        (href_b, uid)
+    b_href_to_status = dict(
+        (href_b, (uid, etag_b))
         for uid, (href_a, etag_a, href_b, etag_b) in status.iteritems()
     )
     # href => {'etag': etag, 'item': optional item, 'uid': uid}
-    list_a = dict(prepare_list(storage_a, a_href_to_uid))
-    list_b = dict(prepare_list(storage_b, b_href_to_uid))
+    list_a = dict(prepare_list(storage_a, a_href_to_status))
+    list_b = dict(prepare_list(storage_b, b_href_to_status))
 
     a_uid_to_href = dict((x['uid'], href) for href, x in list_a.iteritems())
     b_uid_to_href = dict((x['uid'], href) for href, x in list_b.iteritems())
-    del a_href_to_uid, b_href_to_uid
-
-    actions, prefetch_from_a, prefetch_from_b = \
-        get_actions(list_a, list_b, status, a_uid_to_href, b_uid_to_href)
-
-    prefetch(storage_a, list_a, prefetch_from_a)
-    prefetch(storage_b, list_b, prefetch_from_b)
+    del a_href_to_status, b_href_to_status
 
     storages = {
         'a': (storage_a, list_a, a_uid_to_href),
-        'b': (storage_b, list_b, b_uid_to_href),
-        None: (None, None, None)
+        'b': (storage_b, list_b, b_uid_to_href)
     }
 
+    actions = get_actions(storages, status)
+
     for action in actions:
         action(storages, status, conflict_resolution)
 
@@ -130,7 +126,7 @@ def action_update(uid, source, dest):
     return inner
 
 
-def action_delete(uid, source, dest):
+def action_delete(uid, dest):
     def inner(storages, status, conflict_resolution):
         if dest is not None:
             dest_storage, dest_list, dest_uid_to_href = storages[dest]
@@ -175,49 +171,45 @@ def action_conflict_resolve(uid):
     return inner
 
 
-def get_actions(list_a, list_b, status, a_uid_to_href, b_uid_to_href):
-    prefetch_from_a = []
-    prefetch_from_b = []
+def get_actions(storages, status):
     actions = []
-    uids_a = set(x['uid'] for x in list_a.values())
-    uids_b = set(x['uid'] for x in list_b.values())
-    uids_status = set(status)
-    for uid in uids_a.union(uids_b).union(uids_status):
+
+    storage_a, list_a, a_uid_to_href = storages['a']
+    storage_b, list_b, b_uid_to_href = storages['b']
+
+    uids_a = (x['uid'] for x in list_a.itervalues())
+    uids_b = (x['uid'] for x in list_b.itervalues())
+    handled = set()
+    for uid in itertools.chain(uids_a, uids_b, status):
+        if uid in handled:
+            continue
+        handled.add(uid)
+
         href_a = a_uid_to_href.get(uid, None)
         href_b = b_uid_to_href.get(uid, None)
         a = list_a.get(href_a, None)
         b = list_b.get(href_b, None)
         if uid not in status:
-            if uid in uids_a and uid in uids_b:  # missing status
+            if a and b:  # missing status
                 actions.append(action_conflict_resolve(uid))
-            # new item was created in a
-            elif uid in uids_a and uid not in uids_b:
-                prefetch_from_a.append(href_a)
+            elif a and not b:  # new item was created in a
                 actions.append(action_upload(uid, 'a', 'b'))
-            # new item was created in b
-            elif uid not in uids_a and uid in uids_b:
-                prefetch_from_b.append(href_b)
+            elif not a and b:  # new item was created in b
                 actions.append(action_upload(uid, 'b', 'a'))
         else:
             _, status_etag_a, _, status_etag_b = status[uid]
-            if uid in uids_a and uid in uids_b:
+            if a and b:
                 if a['etag'] != status_etag_a and b['etag'] != status_etag_b:
-                    prefetch_from_a.append(href_a)
-                    prefetch_from_b.append(href_b)
                     actions.append(action_conflict_resolve(uid))
                 elif a['etag'] != status_etag_a:  # item was updated in a
-                    prefetch_from_a.append(href_a)
                     actions.append(action_update(uid, 'a', 'b'))
                 elif b['etag'] != status_etag_b:  # item was updated in b
-                    prefetch_from_b.append(href_b)
                     actions.append(action_update(uid, 'b', 'a'))
-                else:  # completely in sync!
-                    pass
-            elif uid in uids_a and uid not in uids_b:  # was deleted from b
-                actions.append(action_delete(uid, None, 'a'))
-            elif uid not in uids_a and uid in uids_b:  # was deleted from a
-                actions.append(action_delete(uid, None, 'b'))
-            # was deleted from a and b
-            elif uid not in uids_a and uid not in uids_b:
-                actions.append(action_delete(uid, None, None))
-    return actions, prefetch_from_a, prefetch_from_b
+            elif a and not b:  # was deleted from b
+                actions.append(action_delete(uid, 'a'))
+            elif not a and b:  # was deleted from a
+                actions.append(action_delete(uid, 'b'))
+            elif not a and not b:  # was deleted from a and b
+                actions.append(action_delete(uid, None))
+
+    return actions
